#!/bin/sh
# $Id: Monitor.tcl,v 1.5 2001/02/26 14:13:23 gdmr Exp $

# Copyright (c) 2000 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Comments are invited.  Please send to gdmr@dcs.ed.ac.uk.

# Scan HP ProCurve switches, comparing what SNMP claims is
# configured against what's in a "definitive" configuration file.

# wish is in a different place on Solaris and Linux, and we can't
# assume that it's in our PATH as we may be running from a cron job.
# This is the standard tcl hack, only with more than one shell line...
#\
case "`uname -sr`" in
#\
	'SunOS 5'*)	WISH=/usr/local/bin/wish ;;
#\
	Linux*)		WISH=/usr/tcl8/bin/wish ;;
#\
	 *)		echo "Don't know where to find tclsh" ; exit ;;
#\
esac
#\
exec $WISH "$0" "$@"
# It's tcl from here on...

# Auto-load from the directory that this script lives in
lappend auto_path [file dirname $argv0]

#
# Get the switch configuration name from the command line
# and any flags
#
proc Usage {} {
   global argv0
   puts stderr "$argv0 \[-Community] host host host ..."
}

proc displayFor {w} {
   global tInfo
   global debug
   if {$debug} {puts stderr "displayFor $w"}
   set m ".$w.m"
   $m delete 0 end
   $m add command -label "Information for \"$tInfo($w,target)\":"
   $m add command -label $tInfo($w,descr)
   $m add command -label $tInfo($w,loc)
   $m add command -label $tInfo($w,upSince)
   set sep 1
   foreach ud $tInfo($w,upDown) {
      if {$debug} {puts stderr "upDown: $ud"}
      if {$sep} {
         $m add separator
         set sep 0
      }
      $m add command -label $ud
   }
   if {!$tInfo($w,downSince)} {
      set tInfo($w,downSince) -1
   }
}

set wIndex 0
proc winpath {} {
   global wIndex
   return "w[incr wIndex]"
}

set tList {}
proc initialise {target community} {
   global tInfo
   global tList
   global debug
   set w [winpath]
   if {$debug} {puts stderr "Initialise: $target $community $w"}
   set tInfo($w,target) $target
   set tInfo($w,upDown) {}
   set tInfo($w,downSince) -1
   set tInfo($w,upSince) "upSince: initialising"
   set tInfo($w,descr) "Descr: initialising"
   set tInfo($w,loc) "Loc: initialising"
   # Set up snmp
   set tInfo($w,snmp) [snmpInit $target $community]
   # Create the target's button and display it
   set tInfo($w,button) \
      [menubutton .$w -text $target -relief sunken \
         -menu .$w.m ]
   menu .$w.m -postcommand "displayFor $w" -tearoff 0
   pack $tInfo($w,button) -fill both -expand true
   # Assume it's up for now
   set tInfo($w,downSince) -1
   # Remember it
   lappend tList $w
}

set debug 0
set verbose 0
set pollInterval 120
foreach a $argv {
   switch -glob -- $a {
      -C*       { regexp "^-C(.*)$" $a all community }
      -debug    { set debug 1 }
      -P*       { regexp "^-P(.*)$" $a all pollInterval }
      -v        -
      -verbose  { set verbose 1 }
      -*	{ Usage ; exit 1 }
      *		{ initialise $a $community }
   }
}

proc scanOne {o} {
   global tInfo
   global verbose
   global debug
   set t $tInfo($o,target)
   set b $tInfo($o,button)
   set s $tInfo($o,snmp)
   set d $tInfo($o,downSince)
   if {$debug} {puts stderr "Scan one: $o -> $t $b $s $d"}
   set now [clock seconds]
   if [catch {snmpUpTime $s} u] {
      # DOWN
      set tInfo($o,upSince) "DOWN"
      if {$d > 0} {
         set downSince [clock format $d]
      } else {
         # State change -> down
         set tInfo($o,downSince) $now
         set downSince [clock format $now]
         lappend tInfo($o,upDown) "DOWN: $downSince"
         if {$debug} {puts stderr "ud: $tInfo($o,upDown)"}
      }
      if {$verbose} {
         $b configure -text "$t: DOWN since $downSince" \
            -background red -activebackground red
      } else {
         $b configure -background red -activebackground red
      }
   } else {
      # up
      set upSince [clock format [expr $now - $u]]
      if {$debug} {puts "Up since $upSince"}
      set tInfo($o,upSince) "up since $upSince"
      if {$d >= 0} {
         # State change -> up
         set tInfo($o,downSince) 0
         set bgc "orange"
         if {$d > 0} {
            lappend tInfo($o,upDown) "up: [clock format $now]"
         }
         if {$debug} {puts stderr "ud: $tInfo($o,upDown)"}
      } else {
         set bgc "green"
      }
      set dl [snmpDescrLoc $s]
      set tInfo($o,descr) [lindex $dl 0]
      set tInfo($o,loc) [lindex $dl 1]
      if {$verbose} {
         $b configure -text "$t: up since $upSince" \
            -background $bgc -activebackground $bgc
      } else {
         $b configure -background $bgc -activebackground $bgc
      }
   }
}

proc scan {} {
   global tList
   global pollInterval
   global verbose
   foreach l $tList {
      scanOne $l
   }
   # set the title bar
   if {$verbose} {
      wm title . "Last scan at [clock format [clock seconds]]"
   }
   # Finally, reschedule
   after [expr {$pollInterval * 1000}] scan
}

# And kick it into life
after 1000 scan
